import m, {Children, Component, Vnode} from "mithril"
import {ButtonColor, Button, ButtonType} from "../base/Button.js"
import {BootIcons} from "../base/icons/BootIcons"
import {LogoutUrl} from "../Header.js"
import {isNewMailActionAvailable, showSupportDialog, showUpgradeDialog, writeInviteMail} from "./NavFunctions"
import {isDesktop, isIOSApp} from "../../api/common/Env"
import {logins} from "../../api/main/LoginController"
import {navButtonRoutes} from "../../misc/RouteChange"
import {getSafeAreaInsetLeft} from "../HtmlUtils"
import {Icons} from "../base/icons/Icons"
import {AriaLandmarks, landmarkAttrs} from "../AriaUtils"
import {attachDropdown} from "../base/Dropdown.js"
import {keyManager} from "../../misc/KeyManager"

type Attrs = {
	openNewWindow(): unknown
}

export class DrawerMenu implements Component<Attrs> {
	view(vnode: Vnode<Attrs>): Children {
		return m(
			"drawer-menu" + landmarkAttrs(AriaLandmarks.Contentinfo, "drawer menu"),
			{
				style: {
					"padding-left": getSafeAreaInsetLeft(),
				},
			},
			m(".flex.col.height-100p.items-center.pt.pb", [
				m(".flex-grow"),
				logins.isGlobalAdminUserLoggedIn() && logins.getUserController().isPremiumAccount()
					? m(Button, {
						icon: () => Icons.Gift,
						label: "buyGiftCard_label",
						click: () => {
							m.route.set("/settings/subscription")
							import("../../subscription/giftcards/PurchaseGiftCardDialog").then(({showPurchaseGiftCardDialog}) => {
								return showPurchaseGiftCardDialog()
							})
						},
						type: ButtonType.ActionLarge,
						colors: ButtonColor.DrawerNav,
					})
					: null,
				isDesktop()
					? m(Button, {
						icon: () => Icons.NewWindow,
						label: "openNewWindow_action",
						click: () => {
							vnode.attrs.openNewWindow()
						},
						type: ButtonType.ActionLarge,
						colors: ButtonColor.DrawerNav,
					})
					: null,
				!isIOSApp() && logins.isUserLoggedIn() && logins.getUserController().isFreeAccount()
					? m(Button, {
						icon: () => BootIcons.Premium,
						label: "upgradePremium_label",
						click: () => showUpgradeDialog(),
						type: ButtonType.ActionLarge,
						colors: ButtonColor.DrawerNav,
					})
					: null,
				m(
					Button,
					attachDropdown(
						{
                            mainButtonAttrs: {
                                label: "showHelp_action",
                                icon: () => BootIcons.Help,
                                type: ButtonType.ActionLarge,
                                click: () => keyManager.openF1Help(),
                                noBubble: true,
                                colors: ButtonColor.DrawerNav,
                            },
                            childAttrs: () => [
                                {
                                    label: "supportMenu_label",
                                    click: () => showSupportDialog(),
                                    type: ButtonType.Dropdown,
                                    colors: ButtonColor.DrawerNav,
                                },
                                {
                                    label: "keyboardShortcuts_title",
                                    click: () => keyManager.openF1Help(true),
                                    type: ButtonType.Dropdown,
                                    colors: ButtonColor.DrawerNav,
                                },
                            ],
                            showDropdown: () => logins.isUserLoggedIn() && logins.getUserController().isPremiumAccount(),
                            width: 300
                        },
					),
				),
				isNewMailActionAvailable() && logins.getUserController().isGlobalAdmin()
					? m(Button, {
						icon: () => BootIcons.Share,
						label: "invite_alt",
						click: () => writeInviteMail(),
						type: ButtonType.ActionLarge,
						colors: ButtonColor.DrawerNav,
					})
					: null,
				logins.isInternalUserLoggedIn()
					? m(Button, {
						icon: () => BootIcons.Settings,
						label: "settings_label",
						click: () => m.route.set(navButtonRoutes.settingsUrl),
						type: ButtonType.ActionLarge,
						colors: ButtonColor.DrawerNav,
					})
					: null,
				m(Button, {
					icon: () => BootIcons.Logout,
					label: "logout_label",
					click: () => m.route.set(LogoutUrl),
					type: ButtonType.ActionLarge,
					colors: ButtonColor.DrawerNav,
				}),
			]),
		)
	}
}